@@ -66,6 +66,32 @@ context("testing lp namespace") {
       expect_true(std::abs(res[i] - ex[i]) < 1e-8);
   }
 
+  test_that("mat_vec_dot_excl_first works") {
+    /*
+     X <- c(0.58, -2.21, 0.82, -1.99, -0.48, -2.21, 1.12, 0.59, 0.62, 0.42, 0.82, 0.59, 0.92, -0.06, 1.36, -1.99, 0.62, -0.06, -0.16, -0.1, -0.48, 0.42, 1.36, -0.1, 0.39)
+     x <- c(-1.82, 0.44, 1.09, 0.05, -0.23, 1.23, 0.11, 0.91, 0.03, 1.48)
+     s <- c(-0.39, -0.36, 0.71, -0.02, -1.62, 1.45, -0.85, 0.43, -1.34, 0.11)
+
+     idx <- c(1:3, 1:2 + 3L + 5L)
+     s[idx] <- s[idx] + matrix(X, 5) %*% x[idx]
+     s[1:3] <- s[1:3] - matrix(X, 5)[1:3, 1:3] %*% x[1:3]
+     dput(s)
+     dput(X[upper.tri(matrix(X, 5), TRUE)])
+     */
+    constexpr size_t const n1(3L),
+                           n2(2L),
+                            n = n1 + n2,
+                          xtr(5L);
+    constexpr double const X [(n * (n + 1)) / 2L] = { 0.58, -2.21, 1.12, 0.82, 0.59, 0.92, -1.99, 0.62, -0.06, -0.16, -0.48, 0.42, 1.36, -0.1, 0.39 },
+                           x [n + xtr]            = { -1.82, 0.44, 1.09, 0.05, -0.23, 1.23, 0.11, 0.91, 0.03, 1.48 },
+                           ex[n + xtr]            = { -1.1601, 0.2802, 2.721, -0.02, -1.62, 1.45, -0.85, 0.43, 2.3364, 3.225 };
+    double res[n + xtr] = { -0.39, -0.36, 0.71, -0.02, -1.62, 1.45, -0.85, 0.43, -1.34, 0.11 };
+
+    lp::mat_vec_dot_excl_first(X, x, x + n1 + xtr, res, res + n1 + xtr, n1, n2);
+    for(size_t i = 0; i < n + xtr; ++i)
+      expect_true(std::abs(res[i] - ex[i]) < 1e-8);
+  }
+
   test_that("rank_one_update works") {
     /*
      X <- c(0.58, -2.21, 0.82, -1.99, -0.48, -2.21, 1.12, 0.59, 0.62, 0.42, 0.82, 0.59, 0.92, -0.06, 1.36, -1.99, 0.62, -0.06, -0.16, -0.1, -0.48, 0.42, 1.36, -0.1, 0.39)
